==> redux-context-test-1.0.4.tgz: package/src/connect.tsx <==
import React, {
  useEffect,
  useContext,
  useReducer,
  Dispatch,
  ComponentType,
} from 'react';
import { AnyAction } from 'redux';
import {
  MapStateToProps, MapDispatchToProps,
} from 'react-redux';
import hoistStatics from 'hoist-non-react-statics';

import Context from './Context.ts';
import { substores } from './Provider.tsx';

type UpdatesReducerState = [any, number]
type ForceComponentUpdateDispatch = (...args: any[]) => any
type ConnectType = {
  (props: any): any;
  WrappedComponent?: React.ComponentType<any>;
  displayName?: string;
}

const initStateUpdates = () => [null, 0];
const storeStateUpdatesReducer = (
  state: UpdatesReducerState,
  action: AnyAction,
) => {
  const [, updateCount] = state;

  return [action.payload, updateCount + 1];
};

const defaultMapDispatchToProps = (dispatch: Dispatch<AnyAction>) => ({ dispatch });
const defaultMapStateToProps = () => ({});

function connect(
  mapStateToProps?: MapStateToProps<any, any>,
  mapDispatchToProps?: MapDispatchToProps<any, any>,
) {
  let shouldHandleStateChanges = Boolean(mapStateToProps);

  if (!mapStateToProps) {
    mapStateToProps = defaultMapStateToProps;
  }
  if (!mapDispatchToProps) {
    mapDispatchToProps = defaultMapDispatchToProps;
  }

  return function WrappedWithConnect(WrappedComponent: ComponentType<any>) {
    const wrappedComponentName = WrappedComponent.displayName
      || WrappedComponent.name || 'Component';
    const displayName = `connect${wrappedComponentName}`;

    const Connect: ConnectType = (props: any) => {
      const { store, name, parentStore } = useContext(Context);

      const state = { ...parentStore!.getState(), ...store!.getState() }
      const stateProps = mapStateToProps!(state, props);
      const dispatchProps = mapDispatchToProps(store?.dispatch, props);

      // 分发dispatch时, 强制更新子组件
      const [, forceComponentUpdateDispatch] = useReducer<
        any,
        null
      >(storeStateUpdatesReducer, null, initStateUpdates);

      useEffect(() => {
        if (!shouldHandleStateChanges) return;

        const forceUpdateFn = () => {
          if (substores.get(name) !== store) return
          (forceComponentUpdateDispatch as ForceComponentUpdateDispatch)({
            type: '@@redux/STORE_UPDATED',
          });
        };
        const unsubscribe = store?.subscribe(forceUpdateFn);

        return () => {
          unsubscribe!();
        };
      }, [store, parentStore]);

      return (
        <WrappedComponent
          {...props}
          {...stateProps}
          {...dispatchProps}
        />
      );
    }

    Connect.WrappedComponent = WrappedComponent;
    Connect.displayName = displayName;

    return hoistStatics(Connect, WrappedComponent);
  };
}

export default connect;

==> redux-context-test-1.0.8.tgz: package/src/connect.tsx <==
import React, {
  useEffect,
  useContext,
  useReducer,
  Dispatch,
  ComponentType,
} from 'react';
import { AnyAction } from 'redux';
import {
  MapStateToProps, MapDispatchToProps,
} from 'react-redux';
import hoistStatics from 'hoist-non-react-statics';

import Context from './Context';
import { substores } from './Provider';

type UpdatesReducerState = [any, number]
type ForceComponentUpdateDispatch = (...args: any[]) => any
type ConnectType = {
  (props: any): any;
  WrappedComponent?: React.ComponentType<any>;
  displayName?: string;
}

const initStateUpdates = () => [null, 0];
const storeStateUpdatesReducer = (
  state: UpdatesReducerState,
  action: AnyAction,
) => {
  const [, updateCount] = state;

  return [action.payload, updateCount + 1];
};

const defaultMapDispatchToProps = (dispatch: Dispatch<AnyAction>) => ({ dispatch });
const defaultMapStateToProps = () => ({});

function connect(
  mapStateToProps?: MapStateToProps<any, any>,
  mapDispatchToProps?: MapDispatchToProps<any, any>,
) {
  let shouldHandleStateChanges = Boolean(mapStateToProps);

  if (!mapStateToProps) {
    mapStateToProps = defaultMapStateToProps;
  }
  if (!mapDispatchToProps) {
    mapDispatchToProps = defaultMapDispatchToProps;
  }

  return function WrappedWithConnect(WrappedComponent: ComponentType<any>) {
    const wrappedComponentName = WrappedComponent.displayName
      || WrappedComponent.name || 'Component';
    const displayName = `connect${wrappedComponentName}`;

    const Connect: ConnectType = (props: any) => {
      const { store, name, parentStore } = useContext(Context);

      const state = { ...parentStore!.getState(), ...store!.getState() }
      const stateProps = mapStateToProps!(state, props);
      const dispatchProps = mapDispatchToProps(store.dispatch, props);

      // 分发dispatch时, 强制更新子组件
      const [, forceComponentUpdateDispatch] = useReducer<
        any,
        null
      >(storeStateUpdatesReducer, null, initStateUpdates);

      useEffect(() => {
        if (!shouldHandleStateChanges) return;

        const forceUpdateFn = () => {
          if (substores.get(name) !== store) return
          (forceComponentUpdateDispatch as ForceComponentUpdateDispatch)({
            type: '@@redux/STORE_UPDATED',
          });
        };
        const unsubscribe = store.subscribe(forceUpdateFn);

        return () => {
          unsubscribe!();
        };
      }, [store, parentStore]);

      return (
        <WrappedComponent
          {...props}
          {...stateProps}
          {...dispatchProps}
        />
      );
    }

    Connect.WrappedComponent = WrappedComponent;
    Connect.displayName = displayName;

    return hoistStatics(Connect, WrappedComponent);
  };
}

export default connect;
